.redux-container-link_color .linkColor { display: inline-block; padding-right: 10px; padding-bottom: 7px; }

.redux-container-link_color .linkColor strong { display: table; margin-bottom: 5px; margin-left: 3px; font-size: 12px; font-weight: normal; color: #999; }

/*# sourceMappingURL=data:application/json;charset=utf8;base64,eyJ2ZXJzaW9uIjozLCJmaWxlIjoicmVkdXgtbGluay1jb2xvci5jc3MiLCJzb3VyY2VzIjpbInJlZHV4LWxpbmstY29sb3Iuc2NzcyJdLCJuYW1lcyI6W10sIm1hcHBpbmdzIjoiQUFBQSxBQUNJLDJCQUR1QixDQUN2QixVQUFVLENBQUMsRUFDUCxPQUFPLEVBQUUsWUFBWSxFQUNyQixhQUFhLEVBQUUsSUFBSSxFQUNuQixjQUFjLEVBQUUsR0FBRyxHQVV0Qjs7QUFkTCxBQU1RLDJCQU5tQixDQUN2QixVQUFVLENBS04sTUFBTSxDQUFDLEVBQ0gsT0FBTyxFQUFFLEtBQUssRUFDZCxhQUFhLEVBQUUsR0FBRyxFQUNsQixXQUFXLEVBQUUsR0FBRyxFQUNoQixTQUFTLEVBQUUsSUFBSSxFQUNmLFdBQVcsRUFBRSxNQUFNLEVBQ25CLEtBQUssRUFBRSxJQUFJLEdBQ2QifQ== */

/*# sourceMappingURL=redux-link-color.css.map */
