.redux-container-select_image {
    margin-top: 2px;
    margin-left: 5px;
    width: 100%;
    margin-bottom: 0;
}

.redux-preview-image {
    max-height: 250px;
    max-width: 250px;
    padding: 5px;
    margin-top: 10px;
    border: 1px solid #e3e3e3;
    background: #f7f7f7;
    -moz-border-radius: 3px;
    -khtml-border-radius: 3px;
    -webkit-border-radius: 3px;
    border-radius: 3px;
}

